/* CanonUtilStatus.c
 * ask for status and read answer from printer
 */
#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <signal.h>
#include <linux/lp.h>
#include <sys/time.h>

#include "CanonUtilGlobal.h"

static char ibuf[1024];

#define OPEN_WRITE_TIMEOUT 1000

static struct  itimerval ti, oti;
static int               delay = OPEN_WRITE_TIMEOUT;

#define RESET_TIMER(ti,oti) { memset((void*)&ti,0,sizeof(ti)); \
                              memset((void*)&oti,0,sizeof(oti)); \
                              setitimer( ITIMER_REAL ,&ti, &oti); \
                            }

#define SET_TIMER(ti,oti,val) { signal(SIGALRM, sigAlarm); \
                                memset((void*)&ti,0,sizeof(ti)); \
                                memset((void*)&oti,0,sizeof(oti)); \
                                ti.it_value.tv_sec  = val/1000; \
                                ti.it_value.tv_usec = (val%1000)*1000; \
                                setitimer( ITIMER_REAL ,&ti, &oti); \
                              }
 
refreshFunction_t refresh = NULL;
#define RE if(refresh)refresh()
 
/*********************************************************************/
/* Function:   sigAlarm                                              */
/*                                                                   */
/*              signal handler, do nothing, but required             */
/*                                                                   */
/* arguments: int code  not used                                     */
/*                                                                   */
/*                                                                   */
/* return:    none                                                   */
/*                                                                   */
/*********************************************************************/

static void sigAlarm(int code)
{
}

/*********************************************************************/
/* Function:   receive                                               */
/*                                                                   */
/*              read from the port and print results to stdout       */
/*                                                                   */
/* arguments: int hd  handle to the opened port                      */
/*                                                                   */
/*                                                                   */
/* return:    none                                                   */
/*                                                                   */
/*********************************************************************/

static int receive(int hd)
{
   int total = 0;
   int rd;
   long dt;
   struct timeval beg, end;

   memset(ibuf,0,sizeof(ibuf));
   gettimeofday(&beg, NULL);
   
   for(;(int)total < (int)sizeof(ibuf) - 1;)
   {
       RE;
       SET_TIMER(ti,oti,100);
       rd = read(hd,ibuf+total,sizeof(ibuf)-total);
       RESET_TIMER(ti,oti);
       if ( rd > 0 )
       {
          total += rd;
          /* if we have got data, all is ok */
          break;
       }
       else
       {
          gettimeofday(&end, NULL);
          dt  = (end.tv_sec  - beg.tv_sec) * 1000;
          dt += (end.tv_usec - beg.tv_usec) / 1000;
          if ( dt > delay )
          {
             break;
          }
       }
   }
#if 0
   if ( total > 0 )
   {
      /* lenght of string */
      int len = ibuf[0] * 256 + ibuf[1];
      if ( len == total )
         printf("%s",ibuf+2);
      else
         printf("%s",ibuf);
   }
   printf("\n");
#endif
   return total== 0?-1:total;
}


/*********************************************************************/
/* Function:   send                                                  */
/*                                                                   */
/*             send the datas contained in the buffer to the printer */
/*             Datas prefixed by % are hex values % is written %%    */
/*                                                                   */
/* arguments: int hdl  handle to the opened port                     */
/*            char *s  data to be passed to the printer              */
/*                                                                   */
/* return:    number of char written                                 */
/*                                                                   */
/*********************************************************************/
  
static int send(int hdl, char *s)
{
   int v;
   int c = 0;
   while (s && *s  )
   {
      RE;
      switch(*s)
      {
         case '%':
            s++;
            if ( *s == '%')
            {
               SET_TIMER(ti,oti,delay);
               if ( write(hdl,s,1)  < 1 )
               {
                  RESET_TIMER(ti,oti);
                  return c;
               }
               RESET_TIMER(ti,oti);
            }   
            else
            {
               if ( *s >= '0' && *s <= '9' )
                  v = (*s -'0')<<4;
               else if ( *s >= 'a' && *s <= 'f' )
                  v = (*s - 'a' + 10)<<4;
               else if ( *s >= 'A' && *s <= 'F' )
                  v = (*s - 'A' + 10)<<4;
               else
                  return 0;
               s++;
               if ( *s >= '0' && *s <= '9' )
                  v |= *s -'0';
               else if ( *s >= 'a' && *s <= 'f' )
                  v |= *s - 'a' + 10;
               else if ( *s >= 'A' && *s <= 'F' )
                  v |= *s - 'A' + 10;
               else
                  return c;

               SET_TIMER(ti,oti,delay);
               if (  write(hdl,&v,1) < 1)
               {
                  RESET_TIMER(ti,oti);
                  return c;
               }
               RESET_TIMER(ti,oti);
            }
            break;
        default:
        {
           SET_TIMER(ti,oti,delay);
           if ( write(hdl,s,1) <1)
           {
              RESET_TIMER(ti,oti);
              return c;
           }
           RESET_TIMER(ti,oti);
        }
      }
      c++;
      s++;
   }
   return c;
}

char *decodeDSC(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncmp(s, "5600", 4) == 0 )
      {
         s += 4;
         ps->printerError = PR_HEAD;
      }
      else if ( strncmp(s, "5700", 4) == 0 )
      {
         s += 4;
         ps->printerError = PR_WASTE;
      }
      else if ( strncmp(s, "NO", 2) == 0 )
      {
         s += 2;
      }
      else
      {
         ps->printerError = PR_ERROR;
         while ( *s && *s != ';' ) s++;
      }
      if ( *s && *s != ';' )
         s++;
   }
   return s;
}

char *decodeDOC(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncmp(s, "1000", 4) == 0 )
      {
         s += 4;
         ps->paperError = 1;
      }
      else if ( strncmp(s, "1300", 4) == 0 )
      {
         s += 4;
         ps->paperError = 1;;
      }
      else if ( strncmp(s, "N0", 2) == 0 )
      {
         s += 2;
      }
      else if ( strncmp(s, "1601", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[BLACK] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1611", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[MAGENTA] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1612", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[CYAN] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1613", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[YELLOW] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1634", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[LMAGENTA] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1635", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[LCYAN] = LEVEL_LOW;
      }
      
      if ( *s && *s != ';' )
         s++;
   }
   return s;
}

char *decodeDWS(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncmp(s, "1900", 4) == 0 )
      {
         s += 4;
      }

      else if ( strncmp(s, "N0", 2) == 0 )
      {
         s += 2;
      }
      else if ( strncmp(s, "1501", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[BLACK] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1511", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[MAGENTA] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1512", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[CYAN] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1513", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[YELLOW] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1534", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[LMAGENTA] = LEVEL_LOW;
      }
      else if ( strncmp(s, "1535", 4) == 0 )
      {
         s += 4;
         ps->colorLevel[LCYAN] = LEVEL_LOW;
      }
      
      if ( *s && *s != ';' )
         s++;
   }
   return s;
}

char *decodeDJS(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncmp(s, "N0", 2) == 0 )
      {
         s += 2;
      }
      else if ( strncmp(s, "CC", 2) == 0 )
      {
         s += 2;
      }
      else
      {
         s += 2;
         ps->busy = 1;
      }
      
      if ( *s && *s != ';' )
         s++;
   }
   return s;
}

char *decodeDBS(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncmp(s, "N0", 2) == 0 )
      {
         s += 2;
      }
      else
      {
         s += 2;
         ps->busy = 1;
      }
      
      if ( *s && *s != ';' )
         s++;
   }
   return s;
}

char *decodeCHD(char *s, printerState_t *ps)
{
   while ( *s && *s != ';' )
   {
      if ( strncpy(ps->installedCartridge, s, 2) )
      {
         s += 2;
      }
      if ( *s && *s != ';' )
      {
         s++;
      }
   }
   return s;
}

void decode(char *s, printerState_t *ps)
{
   while (*s)
   {
      if ( strncmp(s, "DSC:", 4) == 0 )
      {
         s += 4;
         s = decodeDSC(s, ps);
      }
      else if ( strncmp(s, "DOC:", 4) == 0 )
      {
         s += 4;
         s = decodeDOC(s, ps);
      }
      else if ( strncmp(s, "DWS:", 4) == 0 )
      {
         s += 4;
         s = decodeDWS(s, ps);
      }
      else if ( strncmp(s, "DBS:", 4) == 0 )
      {
         s += 4;
         s = decodeDBS(s, ps);
      }
      else if ( strncmp(s, "CHD:", 4) == 0 )
      {
         s += 4;
         s = decodeCHD(s, ps);
      }
      else if ( strncmp(s, "DJS:", 4) == 0 )
      {
         s += 4;
         s = decodeDJS(s, ps);
      }
      else if ( strncmp(s, "PID:", 4) == 0 )
      {
         s += 4;
         while ( *s && *s != ';') s++;
      }
      if ( *s )
         s++;
   }
}

int readStatus(int hd, printerState_t *ps)
{
   int ret = -1;
   if ( hd <= 0 )
   {
      return -1;
   }
   ret = receive(hd);
   if ( ret > 0 )
   {
      decode(ibuf + 2, ps);
      ps->communicationError = 0;
   }
   return ret;
}
 
void getPrinterStatus(char *dev, refreshFunction_t refreshFunction, printerState_t *ps)
{
   int hd;
   
   if ( dev == NULL )
   {
      ps->communicationError = 1;
      return;
   }

   refresh = refreshFunction;
   memset(ps, 0, sizeof(printerState_t));
   ps->communicationError = 1;
   SET_TIMER(ti,oti,100);
   hd = open(dev,O_RDWR);
   RESET_TIMER(ti,oti);

   if ( hd <= 0 )
   {
     ps->communicationError = 1;
      close(hd);
      return;
   }

   if ( send(hd,"%1b[K%24%00%00%1E%00%22SSR=DSC,DOC,DWS,DBS,CHD,DJS,PID;") <= 0 )
   {
      ps->communicationError = 1;
      close(hd);
      return;
   }

   if ( readStatus(hd, ps) < 0 )
   {
      close(hd);
      return;
   }
   
   if ( send(hd, "%1b[K%0b%00%00%1E%00%09SSR=NL;") <= 0 )
   {
      ps->communicationError = 1;
   }
   close(hd);

}


#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
