#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>
#include <time.h>
#include "CanonUtilGlobal.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

static char Init[] = "\033[K\002\000\000\037BJLSTART\n";
static char End[]  = "BJLEND\n";
static char Cmn[]  = "ControlMode=Common\n";


/******************************************************
 * Opening /dev/lp0 is possible if no printer is attached.
 * The write command will the block and we have to react
 * to this. We simply set a alarm handler wich will be
 * armed before of the write call and interrupt it, if
 * writing is not possible.
 * 
 ******************************************************/

static struct  itimerval ti, oti;
typedef void (*signalHandler_t)(int);
static signalHandler_t sig;

/******************************************************
 * Function: _sigAlarm()
 *
 * This signal handler is necessary in order to
 * catch timeout events for file operations 
 * 
 ******************************************************/

#define ALARM_TIMEOUT 500

static void _sigAlarm(int code)
{
#if DEBUG
   fprintf(stderr,"_sigAlarm got\n");
#endif
   ;
}

#define RESET_TIMER(ti,oti) { signal(SIGALRM, sig); \
                              memset((void*)&ti,0,sizeof(ti)); \
                              memset((void*)&oti,0,sizeof(oti)); \
                              setitimer( ITIMER_REAL ,&ti, &oti); \
                            }

#define SET_TIMER(ti,oti,val) { memset((void*)&ti,0,sizeof(ti)); \
                                memset((void*)&oti,0,sizeof(oti)); \
                                ti.it_value.tv_sec  = val/1000; \
                                ti.it_value.tv_usec = (val%1000)*1000; \
                                setitimer( ITIMER_REAL ,&ti, &oti); \
                                sig = signal(SIGALRM, _sigAlarm); \
                              }

/******************************************************
 * Function: sendToLp()
 *
 * Build the command and send it to ths printer via
 * the lp system if available or via the device file 
 * 
 ******************************************************/

int sendToLp(int func, char *printer, char *arg)
{
   char  buffer[1024];
   char  name[1024];
   char *s;
   int   l;
   int   r;
   int   hd;
   FILE *fp;
   memcpy(buffer, Init, sizeof(Init));
   s = buffer + sizeof(Init) - 1 ;

   switch(func)
   {
      case RESET_P:
         strcat(s, "@Reset\n");
      break;
      case NOZZLE_CHECK:
         strcat(s, "@TestPrint=NozzleCheck\n");
      break;
      case CLEAN_B:
         strcat(s, "Cleaning=1K\n");
      break;
      case CLEAN_A:
         strcat(s, "@Cleaning=1All\n");
      break;
      case POW_OFF:
         strcat(s, "@PowerOff\n");
      break;
      case EN_AP_OFF:
         if ( arg == NULL )
            return 1;
         strcat(s,Cmn);
         strcat(s, "AutoPowerOff=");
         strcat(s,arg); //1, 10, 30, 60 Minutes
         strcat(s,"\n");
      break;
      case EN_AP_ON:
         strcat(s,Cmn);
         strcat(s, "AutoPowerOn=Enable\n");
      break;
      case DIS_AP_ON:
         strcat(s,Cmn);
         strcat(s, "AutoPowerOn=Disable\n");
      break;
      case DIS_AP_OFF:
         strcat(s,Cmn);
         strcat(s, "AutoPowerOff=Disable\n");
      break;
      case ADJUST_H:
         strcat(s, "@TestPrint=Auto\n");
      break;
   }
   strcat(s,End);

   l = strlen(s) + sizeof(Init) - 1;
   
   /* string build, open temp file */
   if ( printer && *printer != '-' )
   {
      snprintf(name, sizeof(name), "/tmp/CanonUtil_%d.Prn", getpid());
      if ( (fp = fopen(name,"w")) != NULL )
      {
         fwrite(buffer,l,1,fp);
         fclose(fp);
         if ( fork() == 0)
            execlp("/usr/bin/lpr", "/usr/bin/lpr", "-r", "-P", printer, "-oraw", name ,NULL);
      }
   }
   else if ( device )
   {
      /* we will have a problem if we use the f* function */
      hd = open(device,O_WRONLY);
      if ( hd >= 0 )
      {
         SET_TIMER(ti,oti,ALARM_TIMEOUT);   
         r = write(hd, buffer,l);
         RESET_TIMER(ti,oti);
         close(hd);
         return r > 0 ? 1 : 0;
      }
      else
      {
         return 0;
      }
   }
   return 1;
}

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
