/* preference window
 */
#include <FL/x.H>
#include <FL/Fl.H>
#include <stdlib.h>
#include <string.h>
#include "CanonUtilGlobal.h"
#include "gettext.h"
#include <stdio.h>
#include <pref.h>
#include <getPrinter.h>

#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Choice.H>
#include "CanonUtilFltkUtil.h"
#include "CanonUtilFltkMw.h"
#include "fltkGl.h"

static char *pbLabel[2]  = { "Cancel", "OK" };
static char *pBLabel[3]  = { "PrinterSelection", "DeviceSelection",
                             "lookSelection" };
static char *mainTitleLB = { "CanonBjCUtility" };
static char *lookc[3]    = { "shadow", "normal", "openGl" };

static Fl_Window     *prefWin = 0;
static Fl_Box        *prefTitle;
static Fl_Button     *pbutton[2];
static Fl_Box        *prefBox[3];
static Fl_Choice     *prefChoice[3];
static Fl_Menu_Item  *prefMenu[3];
static Fl_Box        *prefSep;

static int            items[3];

static int oldUseGL;
static int oldButton;

static void lookChange_cb(Fl_Widget *w, void  *data)
{
   int val = prefChoice[2]->value();
   switch (val )
   {
      case 0: // shadow
         useGL        = 0;
         normalButton = 0;
         prefWin->box(FL_FLAT_BOX);
         pbutton[0]->box(FL_PLASTIC_UP_BOX);
         pbutton[1]->box(FL_PLASTIC_UP_BOX);
         setGlWin(prefWin);
         prefWin->redraw();
      break;
      case 1: // normal
         useGL        = 0;
         normalButton = 1;
         prefWin->box(FL_FLAT_BOX);
         pbutton[0]->box(FL_THIN_UP_BOX);
         pbutton[1]->box(FL_THIN_UP_BOX);
         setGlWin(prefWin);
         prefWin->redraw();
      break;
#if HAVE_GL
      case 2: // openGl
         useGL        = 1;
         pbutton[0]->box(FL_UP_BOX);
         pbutton[1]->box(FL_UP_BOX);
         prefBox[0]->box(INVISIBLE_BOX);
         prefBox[1]->box(INVISIBLE_BOX);
         prefBox[2]->box(INVISIBLE_BOX);
         prefWin->box(INVISIBLE_BOX);
         prefWin->damage(FL_DAMAGE_ALL);
         setGlWin(prefWin);
         prefWin->redraw();
      break;
#endif
   }
   redrawMainWin();
}

static void pbuttonCallback(Fl_Widget *w, void  *data)
{
   int i = (int)data;
   if ( i == 0 )
   {
      /* cancel, reset to old values */
      useGL        = oldUseGL;
      normalButton = oldButton;
   }
   else
   {
      if ( printer )
      {
         free(printer);
         printer = NULL;
      }
      if ( pList )
         printer = strdup(prefMenu[0][prefChoice[0]->value()].text);

      if ( device )
      {
         free(device);
         device = NULL;
      }
      if ( dList )
         device  = strdup(prefMenu[1][prefChoice[1]->value()].text);
      if ( look )
      {
         free(look);
         look = NULL;
      }
      look  = strdup(lookc[prefChoice[2]->value()]);
      savePreference();

      redrawMainWin();
   }
   free(prefMenu[0]);
   free(prefMenu[1]);
   free(prefMenu[2]);
   delete prefWin;
}

static void playout()
{
   int tw, th;
   int maxw = 0;
   int mbow = 0, mboh = 0;
   int mchw = 0, mchh = 0;
   int mbuw = 0, mbuh = 0;
   int w, h;
   int x, y;
   int i,j;
   
   getTextSize((Fl_Widget*)prefTitle, &tw,&th);
   
   for ( i = 0; i < 2; i++ )
   {
      getTextSize((Fl_Widget*)pbutton[i], &w, &h);
      if ( w > mbuw ) mbuw = w;
      if ( h > mbuh ) mbuh = h;
   }

   for ( i = 0; i < 3; i++ )
   {
      getTextSize((Fl_Widget*)prefBox[i], &w, &h);
      if ( w > mbow ) mbow = w;
      if ( h > mboh ) mboh = h;

      getTextSize((Fl_Widget*)prefChoice[i], &w, &h);
      j = ((Fl_Menu_*)prefChoice[i])->value();
      if ( w > mchw ) mchw = w;
      if ( h > mchh ) mchh = h;
      y = 0;

      for ( j = 0; j < items[i]-1; j++ )
      {
         getTextWidth(prefChoice[i],(char*)prefMenu[i][j].text,&x);
         if ( x > y ) y = x;
      }
      if ( w + y + 20 > mchw ) mchw = w + y + 20;

      if ( mchh > mboh ) mboh = mchh;
      if ( mchh < mboh ) mchh = mboh;
   }
   
   /* calculate max width */
   maxw = tw + 10;
   w = 5 + mbow + 5 + mchw + 5;
   if ( maxw < w ) maxw = w;
   w = 5 + mbuw + 5 + mbuw + 5;
   if ( maxw < w ) maxw = w;

   /* position and size */
   y = 5;
   prefTitle->resize(5,y,maxw-10,th);
   y += 10 + th;
   for ( i = 0; i < 3; i++ )
   {
      prefBox[i]->resize(5,y,mbow, mboh);
      prefChoice[i]->resize(maxw-5-mchw, y, mchw, mchh);
      y += 10 + mboh;
   }

   prefSep->resize(0,y, maxw,1);
   y += 10;
  
   pbutton[1]->resize(5,y,mbuw, mbuh);
   pbutton[0]->resize(maxw-5-mbuw,y, mbuw, mbuh);
   y += 10 + mbuh;
   prefWin->size(maxw, y);
}

void make_PrefWindow()
{
   list_t *l;
   int i,j, pref;

   getPrinterList();
   getDeviceList();

   oldUseGL  = useGL;;
   oldButton = normalButton;
   
   prefWin = new Fl_Window(10,10/*,100,100*/);
   prefWin->set_modal();
   prefWin->label(prgName);

   { Fl_Box* o = prefTitle = new Fl_Box(5, 5, 10, 10, getText(mainTitleLB));
     o->box(FL_FLAT_BOX);
     o->color(7);
     o->labelsize(18);
     o->labelcolor(1);
     o->align(FL_ALIGN_CLIP);
   }

   for (i=0; i < 3; i++)
   {
      {  Fl_Box* o = prefBox[i] = new Fl_Box(5, 5, 10, 10, getText(pBLabel[i]));
         if ( !useGL )
            o->box(FL_FLAT_BOX);
         o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
      }
      {  Fl_Choice* o = prefChoice[i] = new Fl_Choice(5, 5, 10, 10, "");

         pref = 0;
         if ( i == 0 )
         {
            for ( j = 0, l = pList; l != NULL; j++, l=l->next );
            items[i] = j+1;
            prefMenu[i] = (Fl_Menu_Item*)calloc(sizeof(Fl_Menu_Item),j+1);
            for ( j = 0, l = pList; l != NULL; j++, l=l->next  )
            {
               prefMenu[i][j].text = l->item;
               if ( printer &&strcmp(l->item, printer) == 0 )
                  pref = j;
            }
         }
         else if ( i == 1 )
         {
            for ( j = 0, l = dList; l != NULL; j++, l=l->next );
            items[i] = j+1;
            prefMenu[i] = (Fl_Menu_Item*)calloc(sizeof(Fl_Menu_Item),j+1);
            
            for ( j = 0, l = dList; l != NULL; j++, l=l->next )
            {
               prefMenu[i][j].text = l->item;
               if ( device && strcmp(l->item, device) == 0 )
                  pref = j;
            }
         }
         else if ( i == 2 )
         {
            items[i] = j+1;
#if HAVE_GL            
            prefMenu[2] = (Fl_Menu_Item*)calloc(sizeof(Fl_Menu_Item),4);
            items[i] = 3;
            for ( j = 0; j < 3; j++)
#else
            prefMenu[2] = (Fl_Menu_Item*)calloc(sizeof(Fl_Menu_Item),3);
            items[i] = 2;
            for ( j = 0; j < 2; j++)
#endif
            {
               prefMenu[i][j].text = getText(lookc[j]);
               if ( look && strcmp(lookc[j], look) == 0 )
                  pref = j;
               o->callback(lookChange_cb,0);
            }
         }
         o->menu(prefMenu[i]);
         o->value(pref);
      }
   }

   prefSep = new Fl_Box(5,5,10,5, 0);
   prefSep->box(FL_ENGRAVED_FRAME);

   for ( i = 0; i < 2; i++)
   {
      pbutton[i] = new Fl_Button(0, 0, 0, 0, getText(pbLabel[i]));
      setButton(pbutton[i]);
      pbutton[i]->callback(pbuttonCallback, (void*)i);

      pbutton[i] = new Fl_Button(0, 0, 0, 0, getText(pbLabel[i]));
      setButton(pbutton[i]);
      pbutton[i]->callback(pbuttonCallback, (void*)i);
   }
   prefWin->end();
   playout();
   setGlWin(prefWin);
   prefWin->show();
}
